#ifndef __HUFFMAN__HUFFMANTREE_H
    #define __HUFFMAN__HUFFMANTREE_H

    #include <vector>
    #include <iostream>
    #include <stack>
    #include "sorting.h"
    #include "bitstream.h"

    namespace Huffman
    {

        class HuffmanTree;
        
        class TreeNode
        {
            friend class HuffmanTree;

            protected:
                TreeNode* children[2];
                char character;
                int count;

                static int Compare(TreeNode* a, TreeNode* b);
            public:
                TreeNode(char c = '0');
                virtual ~TreeNode();
        };

        
        class HuffmanTree
        {
            protected:
                TreeNode* root;
                void CreateSymbolTable(TreeNode* node, std::vector<bool> path, std::vector<std::vector<bool> > *symboltable);
                
            public:
                HuffmanTree();
                virtual ~HuffmanTree();

                int ConstructFromStream(std::istream& is);
                void LoadFromStream(std::istream& is);
                void SaveToStream(std::ostream& os);
                
                void Encode(std::istream& is, std::ostream& os);
                void Decode(std::istream& is, int numberofbytes, std::ostream& os);
        };

    }

#endif
